#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2010 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


from ZODB.FileStorage import FileStorage
from ZODB.DB import DB


class TopologicalSort(object):
    new, open, closed = 'new', 'open', 'closed'
    status = None
    result = None

    def __init__(self, nodes):
        """nodes is a dict of (node: [prerequisites])."""
        self.nodes = dict(nodes)

    def visit(self, node):
        if self.status[node] == self.new:
            self.status[node] = self.open
            for dependency in self.nodes[node]:
                if dependency not in self.nodes:
                    raise Exception('Unknown dependency %r (%r)' % (
                        dependency, list(self.nodes)))
                self.visit(dependency)
            self.result.append(node)
            self.status[node] = self.closed
        elif self.status[node] == self.open:
            raise Exception('Cyclic worksheet dependencies:\n %r' % self.status)
        elif self.status[node] == self.closed:
            return

    def __call__(self):
        self.status = dict([(node, self.new)
                            for node in self.nodes])
        self.result = []
        for node in self.nodes:
            self.visit(node)
        return self.result

