#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2010 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import sys


class FakeModule(object):
    ___name = '?'
    def __init__(self, name=None):
        self.___name = name or self.___name
    def __repr__(self):
        return '<FakeModule %r>' % self.___name


class ModulesSnapshot(object):

    def __init__(self):
        self.originals = dict(sys.modules)
        self.faked_attrs = {}

    def mock(self, values):
        """Usage:
        mock({'my.module.Foo': FooClass,
              'my.module.another.GLOBAL_VAL': 1,
              })
        """
        for full_name in sorted(values):
            names = full_name.split('.')
            attr_name = names[-1]
            module_name = '.'.join(names[:-1])
            assert module_name
            self.mock_attr(module_name, attr_name, fake=values[full_name])

    def mock_attr(self, modulename, name, fake=None):
        module = self.get_module(modulename)
        if modulename not in self.faked_attrs:
            self.faked_attrs[modulename] = {
                'injected': [],
                'replaced': {},
                }
        faked = self.faked_attrs[modulename]
        if hasattr(module, name):
            if (name not in faked['injected'] and
                name not in faked['replaced']):
                faked['replaced'][name] = getattr(module, name)
        else:
            faked['injected'].append(name)
        setattr(module, name, fake)

    def mock_module(self, modulename, fake=None):
        if fake is None:
            fake = FakeModule(name=modulename)
        names = modulename.split('.')
        this_name = names[-1]
        parent_name = '.'.join(names[:-1])
        if parent_name:
            self.mock_attr(parent_name, this_name, fake)
        sys.modules[modulename] = fake

    def get_module(self, modulename):
        try:
            sys.modules[modulename]
        except:
            self.mock_module(modulename, fake=FakeModule(name=modulename))
        return sys.modules[modulename]

    def restore(self):
        for module_name, faked in self.faked_attrs.items():
            try:
                module = sys.modules[module_name]
            except:
                continue
            for name in faked['injected']:
                del module.__dict__[name]
            for name, old_value in faked['replaced'].items():
                setattr(module, name, old_value)

        for name in list(sys.modules):
            if name not in self.originals:
                del sys.modules[name]
            else:
                sys.modules[name] = self.originals[name]

