#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from persistent import Persistent


class RelationshipPropertyBase(object):
    annotation_key = None
    def __init__(self, rel_type, my_role, other_role):
        self.rel_type = rel_type
        self.my_role = my_role
        self.other_role = other_role

    def __get__(self, instance, owner):
        if instance is None:
            return self
        else:
            return BoundRelationshipProperty(instance, self.rel_type,
                                             self.my_role, self.other_role,
                                             self.annotation_key)


class BoundRelationshipProperty(object):
    def __init__(self, this, rel_type, my_role, other_role, annotation_key):
        self.this = this
        self.rel_type = rel_type
        self.my_role = my_role
        self.other_role = other_role
        self.annotation_key = annotation_key

    def __nonzero__(self):
        try:
            iter(self).next()
        except StopIteration:
            return False
        else:
            return True

    def __len__(self):
        count = 0
        for i in self:
            count += 1
        return count

    def __iter__(self):
        return iter(getRelatedObjects(self.annotation_key,
                                      self.this, self.other_role,
                                      self.rel_type))


class URIObject(object):
    def __init__(self, uri, name=None, description=''):
        self._uri = uri
        self._name = name
        self._description = description

    uri = property(lambda self: self._uri)
    name = property(lambda self: self._name)
    description = property(lambda self: self._description)

    def __eq__(self, other):
        return self.uri == other.uri

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(self.uri)

    def __repr__(self):
        return '<URIObject %s>' % (self.name or self.uri)


def getRelatedObjects(annotation_key, obj, role, rel_type=None):
    linkset = obj.__annotations__[annotation_key]
    for link in linkset:
        if link.role != role:
            continue
        if (rel_type is not None and
            link.rel_type != rel_type):
            continue
        yield link.target


class LinkSet(Persistent):
    _links = None # OOBTree
    _byrole = None # OOBTree

    def find(self, my_role, target, role, rel_type):
        for link in self:
            if (link.role == role and
                link.rel_type == rel_type and
                link.target is target and
                link.my_role == my_role):
                return link
        raise ValueError(my_role, target, role, rel_type)

    def __iter__(self):
        return iter(self._links.values())


class Link(Persistent):
    my_role = None
    target = None
    role = None
    rel_type = None
    extra_info = None

